var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { matrixProp, CircuitValue, Field, SmartContract, method, Bool, state, State, isReady, Poseidon, Permissions, Mina, Party, } from 'snarkyjs';
export { deploy, submitSolution, getZkAppState, createLocalBlockchain };
await isReady;
class Sudoku extends CircuitValue {
    constructor(value) {
        super();
        this.value = value.map((row) => row.map(Field));
    }
    hash() {
        return Poseidon.hash(this.value.flat());
    }
}
__decorate([
    matrixProp(Field, 9, 9),
    __metadata("design:type", Array)
], Sudoku.prototype, "value", void 0);
export class SudokuZkApp extends SmartContract {
    constructor() {
        super(...arguments);
        this.sudokuHash = State();
        this.isSolved = State();
    }
    init(sudokuInstance) {
        this.sudokuHash.set(sudokuInstance.hash());
        this.isSolved.set(Bool(false));
    }
    submitSolution(sudokuInstance, solutionInstance) {
        let sudoku = sudokuInstance.value;
        let solution = solutionInstance.value;
        console.log("sudoku instance", sudokuInstance);
        console.log("solution instance", solutionInstance);
        // first, we check that the passed solution is a valid sudoku
        // define helpers
        let range9 = Array.from({ length: 9 }, (_, i) => i);
        let oneTo9 = range9.map((i) => Field(i + 1));
        function assertHas1To9(array) {
            oneTo9
                .map((k) => range9.map((i) => array[i].equals(k)).reduce(Bool.or))
                .reduce(Bool.and)
                .assertEquals(true);
        }
        // check all rows
        for (let i = 0; i < 9; i++) {
            let row = solution[i];
            assertHas1To9(row);
        }
        // check all columns
        for (let j = 0; j < 9; j++) {
            let column = solution.map((row) => row[j]);
            assertHas1To9(column);
        }
        // check 3x3 squares
        for (let k = 0; k < 9; k++) {
            let [i0, j0] = divmod(k, 3);
            let square = range9.map((m) => {
                let [i1, j1] = divmod(m, 3);
                return solution[3 * i0 + i1][3 * j0 + j1];
            });
            assertHas1To9(square);
        }
        // next, we check that the solution extends the initial sudoku
        for (let i = 0; i < 9; i++) {
            for (let j = 0; j < 9; j++) {
                let cell = sudoku[i][j];
                let solutionCell = solution[i][j];
                // either the sudoku has nothing in it (indicated by a cell value of 0),
                // or it is equal to the solution
                Bool.or(cell.equals(0), cell.equals(solutionCell)).assertEquals(true);
            }
        }
        // finally, we check that the sudoku is the one that was originally deployed
        let sudokuHash = this.sudokuHash.get(); // get the hash from the blockchain
        this.sudokuHash.assertEquals(sudokuHash); // precondition that links this.sudokuHash.get() to the actual on-chain state
        sudokuInstance.hash().assertEquals(sudokuHash);
        // all checks passed => the sudoku is solved!
        this.isSolved.set(Bool(true));
    }
}
__decorate([
    state(Field),
    __metadata("design:type", Object)
], SudokuZkApp.prototype, "sudokuHash", void 0);
__decorate([
    state(Bool),
    __metadata("design:type", Object)
], SudokuZkApp.prototype, "isSolved", void 0);
__decorate([
    method,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Sudoku]),
    __metadata("design:returntype", void 0)
], SudokuZkApp.prototype, "init", null);
__decorate([
    method,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Sudoku, Sudoku]),
    __metadata("design:returntype", void 0)
], SudokuZkApp.prototype, "submitSolution", null);
// helpers
function createLocalBlockchain() {
    let Local = Mina.LocalBlockchain();
    Mina.setActiveInstance(Local);
    const account = Local.testAccounts[0].privateKey;
    return account;
}
async function deploy(zkAppInstance, zkAppPrivateKey, sudoku, account) {
    let tx = await Mina.transaction(account, () => {
        Party.fundNewAccount(account);
        let sudokuInstance = new Sudoku(sudoku);
        zkAppInstance.deploy({ zkappKey: zkAppPrivateKey });
        zkAppInstance.setPermissions({
            ...Permissions.default(),
            editState: Permissions.proofOrSignature(),
        });
        zkAppInstance.init(sudokuInstance);
    });
    await tx.send().wait();
}
async function submitSolution(sudoku, solution, account, zkAppAddress, zkAppPrivateKey) {
    let tx = await Mina.transaction(account, () => {
        let zkApp = new SudokuZkApp(zkAppAddress);
        zkApp.submitSolution(new Sudoku(sudoku), new Sudoku(solution));
        zkApp.sign(zkAppPrivateKey);
    });
    try {
        await tx.send().wait();
        return true;
    }
    catch (err) {
        return false;
    }
}
function getZkAppState(zkAppInstance) {
    let sudokuHash = fieldToHex(zkAppInstance.sudokuHash.get());
    let isSolved = zkAppInstance.isSolved.get().toBoolean();
    return { sudokuHash, isSolved };
}
function divmod(k, n) {
    let q = Math.floor(k / n);
    return [q, k - q * n];
}
function fieldToHex(field) {
    return BigInt(field.toString()).toString(16);
}
